@@ -9,7 +9,7 @@ use base 'Exporter';
 our @EXPORT_OK = qw/encode decode/;
 
 use version;
-our $VERSION = 'v1.2.0';
+our $VERSION = 'v1.2.1';
 
 use Carp;
 use Config;
@@ -17,7 +17,6 @@ use Scalar::Util qw/blessed/;
 
 use Moo 2.002004; # safer generated code
 use Module::Runtime qw/require_module/;
-use BSON::Types (); # loads types for extjson inflation
 use boolean;
 
 use constant {
@@ -54,6 +53,9 @@ BEGIN {
     *_decode_bson = $class->can("_decode_bson");
 }
 
+# LOAD AFTER XS/PP, so that modules can pick up right version of helpers
+use BSON::Types (); # loads types for extjson inflation
+
 #--------------------------------------------------------------------------#
 # public attributes
 #--------------------------------------------------------------------------#
@@ -266,9 +268,18 @@ sub BUILD {
 
 sub encode_one {
     my ( $self, $document, $options ) = @_;
+    my $type = ref($document);
+
+    Carp::croak "Can't encode scalars" unless $type;
+    # qr// is blessed to 'Regexp';
+    if ( $type eq "Regexp"
+        || !( blessed($document) || $type eq 'HASH' || $type eq 'ARRAY' ) )
+    {
+        Carp::croak "Can't encode non-container of type '$type'";
+    }
 
     $document = BSON::Doc->new(@$document)
-      if ref($document) eq 'ARRAY';
+      if $type eq 'ARRAY';
 
     my $merged_opts = { %$self, ( $options ? %$options : () ) };
 
@@ -617,7 +628,7 @@ BSON - BSON serialization and deserialization
 
 =head1 VERSION
 
-version v1.2.0
+version v1.2.1
 
 =head1 SYNOPSIS
 
