@@ -6,7 +6,7 @@ package BSON::OID;
 # ABSTRACT: BSON type wrapper for Object IDs
 
 use version;
-our $VERSION = 'v1.2.0';
+our $VERSION = 'v1.2.1';
 
 use Carp;
 use Config;
@@ -61,10 +61,10 @@ use namespace::clean -except => 'meta';
     }
     #>>>
 
-    # see if v1.x MongoDB::BSON can do OIDs for us
+    # see if we have XS OID generation
     BEGIN {
-        if ( $INC{'MongoDB'} && MongoDB::BSON->can('generate_oid') ) {
-            *_generate_oid = sub { pack( "H*", MongoDB::BSON::generate_oid() ) };
+        if ( $INC{'BSON/XS.pm'} && BSON::XS->can('_generate_oid') ) {
+            *_generate_oid = \&BSON::XS::_generate_oid;
         }
         else {
             *_generate_oid = \&_packed_oid;
@@ -236,7 +236,7 @@ BSON::OID - BSON type wrapper for Object IDs
 
 =head1 VERSION
 
-version v1.2.0
+version v1.2.1
 
 =head1 SYNOPSIS
 
