@@ -6,12 +6,12 @@ package BSON::PP;
 # ABSTRACT: Pure Perl BSON implementation
 
 use version;
-our $VERSION = 'v1.2.0';
+our $VERSION = 'v1.2.1';
 
 use B;
 use Carp;
 use Config;
-use Scalar::Util qw/blessed looks_like_number refaddr/;
+use Scalar::Util qw/blessed looks_like_number refaddr reftype/;
 use Tie::IxHash;
 
 use BSON::Types ();
@@ -153,8 +153,17 @@ sub _pack_int64 {
     }
 }
 
+sub _reftype_check {
+    my $doc = shift;
+    my $type = ref($doc);
+    my $reftype = reftype($doc);
+    die "Can't encode non-container of type '$type'" unless $reftype eq 'HASH';
+    return;
+}
+
 sub _encode_bson {
     my ($doc, $opt) = @_;
+
     my $refaddr = refaddr($doc);
     die "circular reference detected" if $opt->{_circular}{$refaddr}++;
 
@@ -171,7 +180,7 @@ sub _encode_bson {
       : $doc_type eq 'Tie::IxHash'    ? _ixhash_iterator($doc)
       : $doc_type eq 'BSON::DBRef'    ? _ixhash_iterator( $doc->_ordered )
       : $doc_type eq 'MongoDB::DBRef' ? _ixhash_iterator( $doc->_ordered )
-      :                                 undef;
+      :                                 do { _reftype_check($doc); undef };
 
     my $op_char = defined($opt->{op_char}) ? $opt->{op_char} : '';
     my $invalid =
@@ -429,7 +438,11 @@ sub _encode_bson {
         else {
 
             my $flags = B::svref_2object(\$value)->FLAGS;
-            if ( $flags & B::SVf_NOK() ) {
+            if ( $flags & B::SVf_POK() ) {
+                utf8::encode($value);
+                $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
+            }
+            elsif ( $flags & B::SVf_NOK() ) {
                 $bson .= pack( BSON_TYPE_NAME.BSON_DOUBLE, 0x01, $utf8_key, $value );
             }
             elsif ( $flags & B::SVf_IOK() ) {
@@ -443,10 +456,6 @@ sub _encode_bson {
                     $bson .= pack( BSON_TYPE_NAME . BSON_INT32, 0x10, $utf8_key, $value );
                 }
             }
-            elsif ( $flags & B::SVf_POK() ) {
-                utf8::encode($value);
-                $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
-            }
             elsif ( $] lt '5.010' && $flags & B::SVp_IOK() ) {
                 if ( $value > $max_int64 || $value < $min_int64 ) {
                     croak("BSON can only handle 8-byte integers. Key '$key' is '$value'");
@@ -459,7 +468,9 @@ sub _encode_bson {
                 }
             }
             else {
-                croak("For key '$key', can't encode value '$value'");
+                # Last resort, string
+                utf8::encode($value);
+                $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
             }
 
         }
@@ -795,7 +806,7 @@ BSON::PP - Pure Perl BSON implementation
 
 =head1 VERSION
 
-version v1.2.0
+version v1.2.1
 
 =head1 DESCRIPTION
 
