@@ -6,7 +6,7 @@ package BSON::Time;
 # ABSTRACT: BSON type wrapper for date and time
 
 use version;
-our $VERSION = 'v1.2.0';
+our $VERSION = 'v1.2.1';
 
 use Carp qw/croak/;
 use Config;
@@ -14,6 +14,7 @@ use Time::HiRes qw/time/;
 use Scalar::Util qw/looks_like_number/;
 
 use if !$Config{use64bitint}, 'Math::BigInt';
+use if !$Config{use64bitint}, 'Math::BigFloat';
 
 use Moo;
 
@@ -36,21 +37,47 @@ sub BUILDARGS {
 
     my %args;
     if ( $n == 0 ) {
-        $args{value} = 1000 * time();
+        if ( $Config{use64bitint} ) {
+            $args{value} =  time() * 1000;
+        }
+        else {
+            $args{value} = Math::BigFloat->new(time());
+            $args{value}->bmul(1000);
+            $args{value} = int($args{value});
+        }
     }
     elsif ( $n == 1 ) {
         croak "argument to BSON::Time::new must be epoch seconds, not '$_[0]'"
           unless looks_like_number( $_[0] );
-        $args{value} = 1000 * shift;
+
+        if ( !$Config{use64bitint} && ref($args{value}) ne 'Math::BigInt' ) {
+            $args{value} = Math::BigFloat->new(shift);
+            $args{value}->bmul(1000);
+            $args{value} = int($args{value});
+        }
+        else {
+            $args{value} = 1000 * shift;
+        }
     }
     elsif ( $n % 2 == 0 ) {
         %args = @_;
         if ( defined $args{value} ) {
             croak "argument to BSON::Time::new must be epoch seconds, not '$args{value}'"
               unless looks_like_number( $args{value} ) || overload::Overloaded($args{value});
+
+            if ( !$Config{use64bitint} && ref($args{value}) ne 'Math::BigInt' ) {
+                $args{value} = Math::BigInt->new($args{value});
+            }
         }
         else {
-            $args{value} = 1000 * time();
+            if ( !$Config{use64bitint} && ref($args{value}) ne 'Math::BigInt' ) {
+                $args{value} = Math::BigFloat->new(shift);
+                $args{value}->bmul(1000);
+                $args{value} = int($args{value});
+            }
+            else {
+                $args{value} = 1000 * shift;
+            }
         }
     }
     else {
@@ -60,10 +87,6 @@ sub BUILDARGS {
     # normalize all to integer ms
     $args{value} = int( $args{value} );
 
-    if ( !$Config{use64bitint} && ref($args{value}) ne 'Math::BigInt' ) {
-        $args{value} = Math::BigInt->new($args{value});
-    }
-
     return \%args;
 }
 
@@ -207,7 +230,7 @@ BSON::Time - BSON type wrapper for date and time
 
 =head1 VERSION
 
-version v1.2.0
+version v1.2.1
 
 =head1 SYNOPSIS
 
