@@ -6,7 +6,7 @@ package BSON::Types;
 # ABSTRACT: Helper functions to wrap BSON type classes
 
 use version;
-our $VERSION = 'v1.2.0';
+our $VERSION = 'v1.2.1';
 
 use base 'Exporter';
 our @EXPORT_OK = qw(
@@ -281,7 +281,9 @@ sub bson_string {
 
 sub bson_time {
     return BSON::Time->new unless defined $_[0];
-    return BSON::Time->new( value => 1000 * $_[0] );
+    # Old constructor format handles floating point math right on
+    # 32-bit platforms.
+    return BSON::Time->new( $_[0] );
 }
 
 #pod =func bson_timestamp
@@ -333,7 +335,7 @@ BSON::Types - Helper functions to wrap BSON type classes
 
 =head1 VERSION
 
-version v1.2.0
+version v1.2.1
 
 =head1 SYNOPSIS
 
